/*
#########################################################################
# Subway Map
# ----------
# Copyright (c) 2004 by Dimitri Brukakis
# All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
#########################################################################
*/

#ifndef __DIJKSTRA_H__
#define __DIJKSTRA_H__

#include "baseclasses.h"
#include "Exceptions.h"
#include "SubwayMap.h"
#include "SubwayGraph.h"
#include "RouteFinderBase.h"
#include "SMError.h"
#include "debug.h"

class Dijkstra : public Object
{
 public:
  Dijkstra(SubwayGraph *graph) : _graph(graph), _setNotVisited(NULL), _setLength(0),
	_startStation(NULL), _destStation(NULL),
	_startNodeIndex(-1), _destNodeIndex(-1) {}

  virtual ~Dijkstra() { cleanup(); }

  
  void set_startStation(Station *station) { _startStation=station; }
  void set_destStation(Station *station) { _destStation=station; }

  smerr_t init();
  smerr_t execute();
  smerr_t cleanup();

  smerr_t get_pathNodes(ObjectList& list);

 private:
  smerr_t init_notVisited();
  smerr_t reset_notVisited();
  smerr_t cleanup_notVisited();
  smerr_t add_notVisited(int index);
  SGNode *extract_nearestNode();
  

 protected:
  SubwayGraph *_graph;
  int *_setNotVisited;
  int _setLength;
  Station *_startStation;
  Station *_destStation;

  int _startNodeIndex;
  int _destNodeIndex;
};

#endif
