/*
#########################################################################
# Subway Map
# ----------
# Copyright (c) 2004 by Dimitri Brukakis
# All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
#########################################################################
*/

#ifndef __ROUTEFINDERBASE_H__
#define __ROUTEFINDERBASE_H__


#include "baseclasses.h"
#include "Exceptions.h"
#include "SubwayMap.h"
#include "SMError.h"




/*	==============================================
	Class for storing connection information
	============================================== */

class Connection : public Object
{
 public:
  Connection() : _fromStation(NULL), _toStation(NULL), 
	_useLine(NULL),	_routeOfUsedLine(NULL), _connectionTime(-1),
	_timeToChange(0), _byFeet(0) {}
  Connection(Connection *conn);

  ~Connection() {
	_passedStations.cleanup();
  }

  void set_fromStation(Station *station) { _fromStation=station; }
  void set_toStation(Station *station) { _toStation=station; }
  void set_useLine(Line *line) { _useLine=line; }
  void set_routeOfUsedLine(RouteOfLine *route) { _routeOfUsedLine=route; }
  void set_timeToChange(int ttc) { _timeToChange=ttc; }
  void set_byFeet(bool byFeet) { _byFeet=byFeet; }

  Station *get_fromStation() { return _fromStation; }
  Station *get_toStation() { return _toStation; }
  Line *get_useLine() { return _useLine; }
  RouteOfLine *get_routeOfUsedLine() { return _routeOfUsedLine; }
  int get_timeToChange() { return _timeToChange; }
  bool get_byFeet() { return _byFeet; }

  int calc_time(int recalc=0);

  String& get_directionName();
  
  bool add_passedStation(Station *station);
  Station *first_passedStation();
  Station *next_passedStation();

 protected:
  Station *_fromStation;
  Station *_toStation;
  Line *_useLine;
  RouteOfLine *_routeOfUsedLine;
  int _connectionTime;
  eDirection _direction;
  int _timeToChange;
  bool _byFeet;
  
  ObjectListIterator _passedStationsIt;
  ObjectList _passedStations;

 public:
  friend ostream& operator<<(ostream& os, Connection& s);
  friend ostream& operator<<(ostream& os, Connection* s);

};

/*	==============================================
	Class for storing routes from a start to a
	destination
	============================================== */

class Route : public Object
{
 public:
  Route() : _journeyTime(-1) {}
  Route(Route *route);

  ~Route() {
	_connections.cleanup(1);
  }

  
  void add_connection(Connection *conn);

  Connection *first_connection(ObjectListIterator& it);
  Connection *next_connection(ObjectListIterator& it);
  int get_numConnections();
  bool has_byFeetConnection();

  int calc_journeyTime(int withChanges = 0);

  void copyFrom(Route *route);
  void cleanup() {
	_journeyTime=-1;
	_connections.cleanup(1);
  }

 protected:
  ObjectList _connections;
  int _journeyTime;

 public:
  // -----------------------------
  // dump contents
  // -----------------------------
  friend ostream& operator<<(ostream& os, Route& s);
  friend ostream& operator<<(ostream& os, Route* s);
};


/*	==============================================
	Route list sorting classes
	============================================== */

class SortRouteJourneyTimeUp : public ObjectListSorter
{
 public:
  virtual int compareObjects(Object *route1, Object *route2);
};

class SortRouteConnectionsUp : public ObjectListSorter
{
 public:
  virtual int compareObjects(Object *route1, Object *route2);
};

class SortRouteWeightTime : public ObjectListSorter
{
 public:
  virtual int compareObjects(Object *route1, Object *route2);
};






#endif
