/*
#########################################################################
# Subway Map
# ----------
# Copyright (c) 2004 by Dimitri Brukakis
# All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
#########################################################################
*/

#include "SMXMLSubwayMapParser_Qt.h"





/*	===================================================
	XMLSubwayMapParser virtual methods
	=================================================== */

int SMXMLSubwayMapParser_Qt::initParser()
{

  // Nothing to do, yet.
  return smeOK;
}


int SMXMLSubwayMapParser_Qt::parseDocument(const char *filename)
{
  bool rc;
  QFile xmlFile(filename);

  if(!xmlFile.exists()) {
	return exception(new FileException((const String)"File not found", smeFILE_NOT_FOUND, filename));
  }

  QXmlInputSource source(xmlFile);
  QXmlSimpleReader reader;

  _locator=new QXmlLocator(&reader);

  //  reader.setLocator(locator);
  reader.setContentHandler(this);

  if((rc=reader.parse(source))==FALSE) {
	// something went wrong

	delete _locator;
	_locator=NULL;

	if(_exception) {
	  return _exception->get_extype();
	}
	// returned FALSE but no exception, maybe parsing error
	return exception(new XMLException((const String&)"parsing error", smeXML_PARSING_ERROR, _line, _column));
  }

  delete _locator;
  _locator=NULL;

  return smeOK;
}



/*	===================================================
	Implementation of Qt SAX interface methods
	=================================================== */

bool SMXMLSubwayMapParser_Qt::startDocument()
{
  // cdebug << "startDocument()" << endl;
  return TRUE;
}


bool SMXMLSubwayMapParser_Qt::endDocument()
{
  // cdebug << "endDocument()" << endl;

  return TRUE;
}

bool SMXMLSubwayMapParser_Qt::startElement(const QString& namespaceURI,
										 const QString& qName, const QString& localName,
										 const QXmlAttributes& attrs)
{
  // cdebug << "startElement()" << endl;

  XMLElement element;
  element.set_name((const char *)localName);

  for(int a=0; a<attrs.length(); a++) {
	XMLString aname((const char *)attrs.localName(a));
	XMLString avalue((const char *)attrs.value(a));

	element.add_attribute(aname, avalue);
  }

  // Store document position information
  set_linecolumn(_locator->lineNumber(), _locator->columnNumber());

  if(parse_element(element)!=smeOK)
	return FALSE;	// error information in exception object

  return TRUE;
}

bool SMXMLSubwayMapParser_Qt::endElement(const QString& namespaceURI,
									   const QString& qName, const QString& localName)
{
  // cdebug << "endElement()" << endl;
  XMLElement element;
  element.set_name((const char *)localName);

  // Store document position information
  set_linecolumn(_locator->lineNumber(), _locator->columnNumber());

  if(finish_element(element)!=smeOK)
	return FALSE;
  return TRUE;
}

bool SMXMLSubwayMapParser_Qt::characters(const QString& chr)
{

  return TRUE;
}
