/*
#########################################################################
# Subway Map
# ----------
# Copyright (c) 2004 by Dimitri Brukakis
# All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
#########################################################################
*/

#include "SubwayMap.h"


/*	===========================================================
	Subway Map
	=========================================================== */

smerr_t SubwayMap::add_station(Station *station)
{
  _stations.add(station);

  return smeOK;
}

Station *SubwayMap::first_station(ObjectListIterator& it)
{
  return (Station *)_stations.first(it);
}

Station *SubwayMap::next_station(ObjectListIterator& it)
{
  return (Station *)_stations.next(it);
}

Station *SubwayMap::find_stationById(String& id)
{
  Station *tmpStation;
  ObjectListIterator it;

  tmpStation=(Station *)_stations.first(it);
  while(tmpStation) {
	String& tmpId=tmpStation->get_id();
	if(tmpId==id) return tmpStation;

	tmpStation=(Station *)_stations.next(it);
  }
  return NULL;
}

Station *SubwayMap::find_stationByName(String& name, ObjectListIterator& it)
{
  Station *tmpStation;

  tmpStation=(Station *)_stations.first(it);
  while(tmpStation) {
	String& tmpName=tmpStation->get_name();
	if(tmpName==name) return tmpStation;

	tmpStation=(Station *)_stations.next(it);
  }
  return NULL;
}

Station *SubwayMap::find_stationLikeName(String& name, ObjectListIterator& it)
{
  Station *tmpStation;

  tmpStation=(Station *)_stations.first(it);
  while(tmpStation) {
	String& tmpName=tmpStation->get_name();
	if(tmpName.strnicmp(name, name.length())==0) {
	  // This could be the right station
	  return tmpStation;
	}
	tmpStation=(Station *)_stations.next(it);
  }
  return NULL;
}


smerr_t SubwayMap::sort_stations()
{
  SortStationsByName sorter;

  _stations.bsort(sorter);

  return smeOK;
}

void SubwayMap::remove_station(Station *station)
{
  if(!station) return;
  _stations.remove(station);
}

smerr_t SubwayMap::add_servingLinesToStations()
{
  ObjectListIterator lineIt;
  
  // Add serving line information to each station
  Line *line=NULL;
  line=first_line(lineIt);
  while(line) {
  	ObjectListIterator routeIt;
  	RouteOfLine *routeOfLine=line->first_routeOfLine(routeIt);
  	while(routeOfLine) {
  		ObjectListIterator stopIt;
  		StopAt *stop=routeOfLine->first_stop(stopIt);
  		while(stop) {
  			Station *station=stop->get_station();
  			if(station && !station->has_servingLineId(line->get_id())) {
  				ServingLine *sl=new ServingLine(line->get_id());
  				station->add_servingLine(sl);
  			}
  			stop=routeOfLine->next_stop(stopIt);
  		}
  		routeOfLine=line->next_routeOfLine(routeIt);
  	}
  	line=next_line(lineIt);
  }
  
  return smeOK;	
}
/*	==============================================
	Route list sorting classes
	============================================== */

int SortStationsByName::compareObjects(Object *obj1, Object *obj2)
{
  Station *s1=(Station *)obj1;
  Station *s2=(Station *)obj2;

  if(s1->get_name()>s2->get_name()) return 1;
  if(s1->get_name()<s2->get_name()) return -1;
  return 0;
}

smerr_t SubwayMap::add_line(Line *line)
{
  _lines.add(line);

  return smeOK;
}

Line *SubwayMap::first_line(ObjectListIterator& it)
{
  return (Line *)_lines.first(it);
}

Line *SubwayMap::next_line(ObjectListIterator& it)
{

  return (Line *)_lines.next(it);
}

Line *SubwayMap::find_lineById(String& id)
{
  ObjectListIterator it;
  Line *tmpLine;

  tmpLine=first_line(it);
  while(tmpLine) {
	String& tmpString=tmpLine->get_id();
	if(tmpString==id) return tmpLine;
	tmpLine=next_line(it);
  }
  return NULL;
}

Line *SubwayMap::find_lineByName(String& name)
{
  ObjectListIterator it;
  Line *tmpLine;

  tmpLine=first_line(it);
  while(tmpLine) {
	String& tmpString=tmpLine->get_name();
	if(tmpString==name) return tmpLine;
	tmpLine=next_line(it);
  }
  return NULL;
}

void SubwayMap::cleanup()
{
  _name=NULL;
  _city=NULL;
  _descr=NULL;
  _version=NULL;
  _created=NULL;
  _last_change=NULL;
  _created_by=NULL;

  _stations.cleanup(1);
  _lines.cleanup(1);
}

Line *SubwayMap::get_firstLineOfStation(Station *st, ObjectListIterator& it)
{
  ServingLine *servLine=st->first_servingLine(it);
  if(servLine==NULL) return NULL;

  Line *line=find_lineById(servLine->get_id());

  return line;
}

Line *SubwayMap::get_nextLineOfStation(Station *st, ObjectListIterator& it)
{
  ServingLine *servLine=st->next_servingLine(it);
  if(servLine==NULL) return NULL;

  Line *line=find_lineById(servLine->get_id());
  
  return line;
}

int SubwayMap::get_numStationsOnAllLines()
{
  int count=0;

  // count the number of stations on all routes of all lines
  ObjectListIterator lineIt;
  Line *line=first_line(lineIt);
  while(line) {

	ObjectListIterator routeIt;
	RouteOfLine *routeOfLine=line->first_routeOfLine(routeIt);
	while(routeOfLine) {
	  count+=routeOfLine->get_numStops();
	  routeOfLine=line->next_routeOfLine(routeIt);
	}

	line=next_line(lineIt);
  }
  return count;
}


/*	===========================================================
	Subway Map stations
	=========================================================== */

void Station::add_servingLine(ServingLine *line) 
{
	_servingLines.add(line);
}

void Station::cleanup()
{
  _name=NULL;
  _id=NULL;

  _servingLines.cleanup(1);
}

ServingLine *Station::first_servingLine(ObjectListIterator& it)
{
  return (ServingLine *)_servingLines.first(it);
}

ServingLine *Station::next_servingLine(ObjectListIterator& it)
{
  return (ServingLine *)_servingLines.next(it);
}

int Station::has_servingLine(ServingLine& testLine)
{
  ObjectListIterator it;
  ServingLine *servingLine=first_servingLine(it);
  while(servingLine) {
	String& id1=servingLine->get_id();
	String& id2=testLine.get_id();

	if(id1==id2) 
	  return 1;	// found it
	servingLine=next_servingLine(it);
  }
  // not found
  return 0;
}	
int Station::has_servingLine(ServingLine *testLine)
{
  return has_servingLine(*testLine);
}

int Station::has_servingLineId(String& id)
{
  ObjectListIterator it;
  ServingLine *servingLine=first_servingLine(it);
  while(servingLine) {
	String& id2=servingLine->get_id();
	if(id==id2) 
	  return 1; // got it
	servingLine=next_servingLine(it);
  }
  // not found
  return 0;
}

ServingLine *Station::match_servingLine(Station *station)
{
  ServingLine *sl1=NULL;
  ObjectListIterator it1;

  // find a matching line for both stations

  sl1=first_servingLine(it1);
  while(sl1) {
	if(station->has_servingLine(sl1))
	  return sl1;

	sl1=next_servingLine(it1);
  }
  return NULL;
}

/*	===========================================================
	Operation times
	=========================================================== */
void OperationTime::set_always(EDay day)
{
  if(day==everyday) {
	for(int d=0; d<7; d++)
	  _optimes[d]=0xffffff;
	_always=true;
  } else {
	_optimes[static_cast<unsigned int>(day)]=0xffffff;
  }

  _never=false;
}

void OperationTime::set_never(EDay day)
{
  if(day==everyday) {
	for(int d=0; d<7; d++)
	  _optimes[d]=0;
	_never=true;
  } else {
	_optimes[static_cast<unsigned int>(day)]=0;
  }
  _always=false;
}

smerr_t OperationTime::set_ophour(EDay day, unsigned int hour, bool operates)
{
  unsigned int bitmap=0;

  if(_always || _never) { // initialized to always
	// initialize bitmaps
	set_never(everyday);

	_always=false;
	_never=false;
  }

  bitmap=hour_to_bit(hour);
  if(bitmap==0)
	return smeROUTE_ILLEGAL_HOUR;

  if(day>sunday)
	return smeROUTE_ILLEGAL_DAY;

  if(operates) {
	_optimes[static_cast<unsigned int>(day)] |= bitmap; // set bit
  } else {
	_optimes[static_cast<unsigned int>(day)] ^= bitmap; // unset set bit
  }
  return smeOK;
}

smerr_t OperationTime::set_ophours(EDay day, unsigned int *hours, int numhours, bool operates)
{
  for(int i=0; i<numhours; i++) {
	set_ophour(day, hours[i], operates);
  }
  return smeOK;
}


bool OperationTime::operates_at(EDay day, unsigned int hour)
{
  if(day>sunday || hour>23)
	return false;

  unsigned int bitmap=hour_to_bit(hour);

  if(_optimes[static_cast<unsigned int>(day)] & bitmap)
	return true;

  return false;
}

bool OperationTime::operates_on(EDay day)
{
  if(_optimes[static_cast<unsigned int>(day)])
	return true;

  return false;
}

unsigned int OperationTime::hour_to_bit(unsigned int hour)
{
  unsigned int bitmap=1; // init:  b0000...0001

  if(hour>23) // only 0..23 (0:00..23:00)
	return 0;

  // left-shift hour
  bitmap=bitmap<<hour;

  return bitmap;
}

void OperationTime::dayname(EDay day, String& str)
{
  char *days[7]={
	"Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"
  };

  str=days[static_cast<int>(day)];
}

void OperationTime::hourstr(EDay day, String& str)
{
  char hourstr[80];

  memset(hourstr, 0, 80);

  for(int h=0; h<24; h++) {
	if(operates_at(day, h)) {
	  char tmpstr[4];
	  sprintf(tmpstr, "%02d ", h);
	  strcat(hourstr, tmpstr);
	} else {
      strcat(hourstr, "   ");
	}
  }
  str=hourstr;
}

bool OperationTime::convert_time_t(EDay& day, unsigned int& hour, time_t t)
{
  unsigned char dayconv[7]={
	6,0,1,2,3,4,5
  };
  // Converts a given time to day and hour.
  // A time_t of 0 uses actual time (now).
  time_t itime=(t!=0) ? t : time(NULL);

  struct tm tm;
  localtime_r(&itime, &tm);

  hour=tm.tm_hour;
  day=static_cast<EDay>(dayconv[tm.tm_wday]);

  return true;
}

/*	===========================================================
	Subway Map Lines
	=========================================================== */
void Line::add_routeOfLine(RouteOfLine *route)
{
  _routesOfLine.add(route);
}

RouteOfLine *Line::get_routeOfLine()
{
  ObjectListIterator it;
  return (RouteOfLine *)_routesOfLine.first(it);
}

RouteOfLine *Line::first_routeOfLine(ObjectListIterator& it)
{
  return (RouteOfLine *)_routesOfLine.first(it);
}

RouteOfLine *Line::next_routeOfLine(ObjectListIterator& it)
{
  return (RouteOfLine *)_routesOfLine.next(it);
}

RouteOfLine *Line::get_routeOfLineWithStation(Station *station)
{
  ObjectListIterator it;
  RouteOfLine *tmpRoute=first_routeOfLine(it);
  while(tmpRoute) {
	ObjectListIterator stationIt;
	StopAt *tmpStop;
	if((tmpStop=tmpRoute->find_stopByStation(station, stationIt))!=NULL) {
	  return tmpRoute;
	}
	tmpRoute=next_routeOfLine(it);
  }
  return NULL;
}

void Line::cleanup()
{
  _name=NULL;
  _id=NULL;

  _routesOfLine.cleanup(1);

}

bool Line::operates_at(OperationTime::EDay day, unsigned int hour)
{
  ObjectListIterator it;
  RouteOfLine *routeOfLine=first_routeOfLine(it);
  bool operates=false;

  while(routeOfLine && !operates) {
	if(routeOfLine->operates_at(day, hour))
	  operates=true;

	routeOfLine=next_routeOfLine(it);
  }

  return operates;
}
/*	===========================================================
	Subway Map Route
	=========================================================== */
void RouteOfLine::add_stop(StopAt* stop)
{
  _stops.add(stop);
}


StopAt *RouteOfLine::find_stopByStationId(String& id, ObjectListIterator& it)
{
  StopAt *stop=first_stop(it);

  while(stop) {
	Station *station=stop->get_station();
	if(station) { // safety
	  String& stationId=station->get_id();
	  if(stationId==id) { // found it
		return stop;
	  }
	}
	stop=next_stop(it);
  }

  return NULL;
}

StopAt *RouteOfLine::find_stopByStation(Station *station, ObjectListIterator& it)
{
  String& string=station->get_id();

  return find_stopByStationId(string, it);
}

StopAt *RouteOfLine::first_stop(ObjectListIterator& it)
{
  return (StopAt *)_stops.first(it);
}

StopAt *RouteOfLine::next_stop(ObjectListIterator& it)
{
  return (StopAt *)_stops.next(it);
}

StopAt *RouteOfLine::prev_stop(ObjectListIterator& it)
{
  return (StopAt *)_stops.prev(it);
}

bool RouteOfLine::hasStation(Station *station)
{
  ObjectListIterator it;
  StopAt *tmpStop=NULL;

  tmpStop=find_stopByStation(station, it);
  if(tmpStop) return true;
  return false;
}

eDirection RouteOfLine::get_routeDirection(Station *startStation, Station *destStation)
{
  ObjectListIterator it;
  Station *station=NULL;

  StopAt *stopAtStart=find_stopByStation(startStation, it);
  if(!stopAtStart) return not_found;

  // Search in forward direction
  StopAt *stopAtTmp=stopAtStart;
  ObjectListIterator tmpIt=it;

  stopAtTmp=next_stop(tmpIt);
  while(stopAtTmp) {
	station=stopAtTmp->get_station();
	if(station==destStation) return forward;
	stopAtTmp=next_stop(tmpIt);
  }

  // Not found, search backwards
  stopAtTmp=stopAtStart;
  tmpIt=it;
  stopAtTmp=prev_stop(tmpIt);
  while(stopAtTmp) {
	station=stopAtTmp->get_station();
	if(station==destStation) return backwards;
	stopAtTmp=prev_stop(tmpIt);
  }

  // not found
  return not_found;
}


void RouteOfLine::cleanup()
{
  _name=NULL;
  _service_start=NULL;
  _service_end=NULL;
  _startstation=NULL;
  _endstation=NULL;

  _stops.cleanup(1);
}

bool RouteOfLine::operates_at(OperationTime::EDay day, unsigned int hour)
{
  return _optime.operates_at(day, hour);
}

/*	===========================================================
	Subway Map StopAt
	=========================================================== */

void StopAt::add_changeToLine(ChangeToLine *ctl)
{
  _changeToLine.add(ctl);
}
ChangeToLine *StopAt::first_changeToLine(ObjectListIterator& it)
{
  return (ChangeToLine *)_changeToLine.first(it);
}

ChangeToLine *StopAt::next_changeToLine(ObjectListIterator& it)
{
  return (ChangeToLine *)_changeToLine.next(it);
}

void StopAt::add_changeToStation(ChangeToStation *ctl)
{
  _changeToStation.add(ctl);
}
ChangeToStation *StopAt::first_changeToStation(ObjectListIterator& it)
{
  return (ChangeToStation *)_changeToStation.first(it);
}

ChangeToStation *StopAt::next_changeToStation(ObjectListIterator& it)
{
  return (ChangeToStation *)_changeToStation.next(it);
}


/*	===========================================================
	Methods for dumping the subway map network
	=========================================================== */

/*	--------------
	Subway map
	-------------- */
ostream& operator<<(ostream& os, SubwayMap& s)
{
  os << "=========" << endl;
  os << "SubwayMap" << endl;
  os << "=========" << endl;

  os << endl;
  os << "Info:" << endl;
  os << "-----" << endl;
  os << "Name:    " << s._name << endl;
  os << "City:    " << s._city << endl;
  os << "Descr:   " << s._descr << endl;
  os << "Created: " << s._created << " by " << s._created_by << endl;
  os << "Version: " << s._version << " from " << s._last_change << endl;
  os << endl;

  os << "Stations:" << endl;
  os << "---------" << endl;

  ObjectListIterator it;
  Station *station=(Station *)s._stations.first(it);
  while(station) {
	os << station;
	station=(Station *)s._stations.next(it);
  }
  os << endl;

  os << "Lines:" << endl;
  os << "------" << endl;

  Line *line=(Line *)s._lines.first(it);
  while(line) {
	os << line;
	line=(Line *)s._lines.next(it);
  }
  os << endl;

  return os;
}

ostream& operator<<(ostream& os, SubwayMap* s)
{
  os << *s;
  return os;
}


SMXMLWriter& operator<<(SMXMLWriter& os, SubwayMap& s)
{
  // ------------------------------
  // Write document and info tag
  // ------------------------------
  os << "<subway_map>" << endl;
  os << "<info name=\"" << s._name << "\" city=\"" << s._city << "\"" << endl;
  os << "      descr=\"" << s._descr << "\"" << endl;
  os << "      created_by=\"" << s._created_by << "\"" << endl;
  os << "      created=\"" << s._created << "\"" << endl;
  os << "      version=\"" << s._version << "\"" << endl;
  os << "      last_change=\"" << s._last_change << "\">" << endl;
  os << "</info>" << endl << endl;

  // ------------------------------
  // Write station section
  // ------------------------------

  os << "<stations>" << endl;
  ObjectListIterator it;
  Station *station=s.first_station(it);

  while(station) {
	os << station;
	station=s.next_station(it);
  }
  os << "</stations>" << endl;
  os << endl;

  // ------------------------------
  // Write lines section
  // ------------------------------

  os << "<lines>" << endl;
  Line *line=s.first_line(it);
  while(line) {
	os << line;
	line=s.next_line(it);
  }
  os << "</lines>" << endl;
 
  os << "</subway_map>" << endl;

  return os;
}

SMXMLWriter& operator<<(SMXMLWriter& os, SubwayMap* s)
{
  os << *s;
  return os;
}

/*	--------------
	Stations
	-------------- */

ostream& operator<<(ostream& os, Station& s)
{
  os << s._name << " ID=" << s._id;
 
  os << endl;

  ObjectListIterator it;
  ServingLine *servingLine=(ServingLine *)s._servingLines.first(it);
  while(servingLine) {
	os << "       Line "<< servingLine << endl;
	servingLine=(ServingLine *)s._servingLines.next(it);
  }

  return os;
}


ostream& operator<<(ostream& os, Station* s)
{
  os << *s;
  return os;
}

SMXMLWriter& operator<<(SMXMLWriter& os, Station& s)
{
  os << "  <station id=\"" << s._id << "\" name=\"" << s._name << "\">";
  ObjectListIterator it;
  ServingLine *servingLine=s.first_servingLine(it);
  while(servingLine) {
	os << servingLine;
	servingLine=s.next_servingLine(it);
  }
  os << "</station>" << endl;

  return os;
}

SMXMLWriter& operator<<(SMXMLWriter& os, Station* s)
{
  os << *s;
  return os;
}

ostream& operator<<(ostream& os, ServingLine& s)
{
  os << s._id;
  return os;
}
ostream& operator<<(ostream& os, ServingLine* s)
{
  os << *s;
  return os;
}

SMXMLWriter& operator<<(SMXMLWriter& os, ServingLine& s)
{
  os << "<sl id=\"" << s._id << "\"/>";
  return os;
}

SMXMLWriter& operator<<(SMXMLWriter& os, ServingLine* s)
{
  os << *s;
  return os;
}

/*	--------------
	Lines
	-------------- */
ostream& operator<<(ostream& os, Line& s)
{
  os << "Line " << s._name << " (" << s._id << ")" << endl;

  ObjectListIterator it;
  RouteOfLine *routeOfLine=(RouteOfLine *)s._routesOfLine.first(it);
  while(routeOfLine) {
	os << routeOfLine;
	routeOfLine=(RouteOfLine *)s._routesOfLine.next(it);
  }

  os << endl;

  return os;
}
ostream& operator<<(ostream& os, Line* s)
{
  return os << *s;
}

SMXMLWriter& operator<<(SMXMLWriter& os, Line& s)
{
  os << "  <line id=\"" << s._id << "\" name=\"" << s._name << "\" class=\"" << s._class << "\">" << endl;

  ObjectListIterator it;
  RouteOfLine *routeOfLine=s.first_routeOfLine(it);
  while(routeOfLine) {
	os << routeOfLine;
	routeOfLine=s.next_routeOfLine(it);
  }
  os << "  </line>" << endl;
  return os;
}
SMXMLWriter& operator<<(SMXMLWriter& os, Line* s)
{
  os << *s;
  return os;
}


ostream& operator<<(ostream& os, RouteOfLine& s)
{
  os << "   Route from " << s._startstation << " to " << s._endstation << endl;
  os << "   Operation times:" << endl;
  os << s._optime << endl;
  os << "   Stops: " << endl;

  ObjectListIterator it;
  StopAt *stop=(StopAt *)s._stops.first(it);
  while(stop) {
	os << stop;
	stop=(StopAt *)s._stops.next(it);
  }
  os << endl;
  return os;
}
ostream& operator<<(ostream& os, RouteOfLine* s)
{
  return os << *s;
}


SMXMLWriter& operator<<(SMXMLWriter& os, RouteOfLine& s)
{
  os << "    <route name=\"" << s._name << "\"" << endl;
  os << "           startstation=\"" << s._startstation 
	 << "\" endstation=\"" << s._endstation << "\"" << endl;
  if(s.get_service_start()!=NULLString && s.get_service_end()!=NULLString) {
	os << "           service_start=\"" << s._service_start << "\" service_end=\"" 
	   << s._service_end << "\"";
  }
  if(s.get_oneway()) {
	os << endl;
	os << "           oneway=\"true\"";
  }
  os << ">" << endl;

  ObjectListIterator it;
  StopAt *stopAt=s.first_stop(it);
  while(stopAt) {
	os << stopAt;
	stopAt=s.next_stop(it);
  }
  os << "    </route>" << endl;

  return os;
}
SMXMLWriter& operator<<(SMXMLWriter& os, RouteOfLine* s)
{
  os << *s;
  return os;
}

ostream& operator<<(ostream& os, StopAt& s)
{
  os << "    " << s._station->get_name() << " ttn=" << s._timeToNext << " ttp=" << s._timeToPrev;

  ObjectListIterator it;
  ChangeToLine *ctl=s.first_changeToLine(it);
  if(ctl) {
	os << "  Change: ";
  }
  while(ctl) {
	os << ctl->get_id();
	if(ctl->get_fromDir()==forward)
	  os << "[from_dir >]";
	else if(ctl->get_fromDir()==backwards)
	  os << "[< from_dir]";

	if(ctl->get_toDir()==forward)
	  os << "[to_dir >]";
	else if(ctl->get_toDir()==backwards)
	  os << "[< to_dir]";

	os << " ";
	ctl=s.next_changeToLine(it);
  }

  ChangeToStation *cts=s.first_changeToStation(it);
  while(cts) {
	os << cts->get_id();
	if(cts->get_fromDir()==forward)
	  os << "[from_dir >]";
	else if(cts->get_fromDir()==backwards)
	  os << "[< from_dir]";

	if(cts->get_toDir()==forward)
	  os << "[to_dir >]";
	else if(cts->get_toDir()==backwards)
	  os << "[< to_dir]";

	os << " ";
	cts=s.next_changeToStation(it);
  }
  
  os << endl;
  return os;
}
ostream& operator<<(ostream& os, StopAt* s)
{
  return os << *s;
}

SMXMLWriter& operator<<(SMXMLWriter& os, StopAt& s)
{
  os << "     <stop at=\"" << s._station->get_id() << "\" ";
  os << "ttn=\"" << s._timeToNext << "\" ttp=\"" << s._timeToPrev << "\">";

  ObjectListIterator it;
  ChangeToLine *ctl=s.first_changeToLine(it);
  while(ctl) {
	os << ctl;
	ctl=s.next_changeToLine(it);
  }

  ChangeToStation *cts=s.first_changeToStation(it);
  while(cts) {
	os << cts;
	cts=s.next_changeToStation(it);
  }

  os << "</stop>" << endl;

  return os;
}

SMXMLWriter& operator<<(SMXMLWriter& os, StopAt* s)
{
  return os << *s;
}

SMXMLWriter& operator<<(SMXMLWriter& os, ChangeToLine& s)
{
  os << "<ctl id=\"" << s._id << "\" ttc=\"" << s._timeToChange << "\"";
  if(s._fromDir==forward)
	os << " from_dir=\"forw\"";
  if(s._fromDir==backwards)
	os << " from_dir=\"backw\"";
  if(s._toDir==forward)
	os << " to_dir=\"forw\"";
  if(s._toDir==backwards)
	os << " to_dir=\"backw\"";


  os << "/>";
  return os;
}
SMXMLWriter& operator<<(SMXMLWriter& os, ChangeToLine* s)
{
  return os << *s;
}

SMXMLWriter& operator<<(SMXMLWriter& os, ChangeToStation& s)
{
  os << "<cts id=\"" << s._id << "\" ttc=\"" << s._timeToChange << "\"";
  if(s._fromDir==forward)
	os << " from_dir=\"forw\"";
  if(s._fromDir==backwards)
	os << " from_dir=\"backw\"";
  if(s._toDir==forward)
	os << " to_dir=\"forw\"";
  if(s._toDir==backwards)
	os << " to_dir=\"backw\"";

  os << "/>";
  return os;
}
SMXMLWriter& operator<<(SMXMLWriter& os, ChangeToStation* s)
{
  return os << *s;
}


//	-----------------
//	OperationTime
//	-----------------
ostream& operator<<(ostream& os, OperationTime& s)
{
  for(int i=0; i<7; i++) {
	if(s.operates_on(static_cast<OperationTime::EDay>(i))) {
	  String daystr;
	  String hourstr;

	  s.dayname(static_cast<OperationTime::EDay>(i), daystr);
	  s.hourstr(static_cast<OperationTime::EDay>(i), hourstr);

	  os << "   " << (const char *)daystr << " ";
	  os << (const char *)hourstr;
	  os << endl;
	}
  }
  return os;
}

ostream& operator<<(ostream& os, OperationTime *s)
{
  return os<<*s;
}
SMXMLWriter& operator<<(SMXMLWriter& os, OperationTime& s)
{
  return os;
}

SMXMLWriter& operator<<(SMXMLWriter& os, OperationTime* s)
{
  return os<<*s;
}


