/*
#########################################################################
# Subway Map
# ----------
# Copyright (c) 2004 by Dimitri Brukakis
# All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
#########################################################################
*/


#include "AltConnectionDialog.h"
#include "images.h"



AltConnectionDialog::AltConnectionDialog(QWidget *parent, const char *name, bool modal, WFlags fl)
  : AltConnectionDialogBase(parent, name, modal, fl), _startStation(NULL), _destStation(NULL)    
{
}


AltConnectionDialog::~AltConnectionDialog()
{
}


void AltConnectionDialog::start_findConnections()
{
  smerr_t rc;
  
  if(!_startStation || !_destStation || !_subwayMap)
  	return;
  	

  _routeFinder.use_subwayMap(_subwayMap);
  _routeFinder.set_startStation(_startStation);
  _routeFinder.set_destStation(_destStation);
  rc=_routeFinder.findAllDirectConnections();
  if(rc!=smeOK) {
  	// show error dialog...
  	return;
  }  	

  _fromLabel->setText((const char *)_startStation->get_name());
  _toLabel->setText((const char *)_destStation->get_name());

  AltConnectionItem *lastItem=NULL;
  
  ObjectListIterator routeIt;
  Route *route=_routeFinder.first_routeFound(routeIt);


  while(route) {
  	// There is only one connection, because it's a direct route
  	ObjectListIterator connIt;
  	Connection *conn=route->first_connection(connIt);
	int timeToGo=conn->calc_time();
	
	Line *line=conn->get_useLine();
	
	if(line) {
		QPixmap *pixLine=NULL;
		if(_iconStore) {
			pixLine=_iconStore->find_iconByLineId(line->get_id());
		}

		AltConnectionItem *item=new AltConnectionItem(_linesList, lastItem, pixLine, 
											(const char*)line->get_name(),
											(const char*)conn->get_directionName(),
											timeToGo);
		lastItem=item;		
	}

  	route=_routeFinder.next_routeFound(routeIt);

  }
}

void AltConnectionDialog::cleanup()
{
  _linesList->clear();
  _routeFinder.cleanup();
}


AltConnectionItem::AltConnectionItem(QListView *parent, QListViewItem *lastItem, QPixmap *pixmap, const char *name, const char *direction, int timeToGo) : QListViewItem(parent, lastItem)
{
	if(pixmap) {
		setPixmap(0, *pixmap);
	} else {
		setText(0, QString(name));
	}
	
	QString tmpStr;
	tmpStr=" [";
	tmpStr+=direction;
	tmpStr+="] ";
	tmpStr+=QString::number(timeToGo);
	tmpStr+="min.";
	
	setText(1, tmpStr);
}

AltConnectionItem::~AltConnectionItem()
{
}

