/*
#########################################################################
# Subway Map
# ----------
# Copyright (c) 2004 by Dimitri Brukakis
# All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
#########################################################################
*/


#include <qlabel.h>
#include <qlistview.h>
#include <qcombobox.h>

#include "LineInfoDialog.h"
#include "images.h"

LineInfoDialog::LineInfoDialog( QWidget* parent, const char* name, bool modal, WFlags fl ):
  LineInfoDialogBase(parent, name, modal, fl)
{

/*
  connect((QObject*)_buttonNow, SIGNAL(clicked()), this, SLOT(setTimeNow()));
  connect((QObject*)_comboDay, SIGNAL(activated(int)), this, SLOT(hasSelected(int)));
  connect((QObject*)_comboTime, SIGNAL(activated(int)), this, SLOT(hasSelected(int)));
*/

  connect((QObject*)_routeCombo, SIGNAL(activated(int)), this, SLOT(selectedRouteOfLine(int)));

  _routeOfLineView->setSorting(-1);
  _operationTimeView->setSorting(-1);
}


LineInfoDialog::~LineInfoDialog()
{

}

void LineInfoDialog::set_lineAndRoute(Line *line, RouteOfLine *routeOfLine)
{
	QPixmap *pixLine=NULL;
	QString strLine;
	int index=0;
	int indexOfCurrentRoute=0;
		
	if(!line)
		return;
		
	_line=line;
	
	strLine=(const char *)line->get_name();
	_lineLabel->setText(strLine);
	if(_iconStore) {
		pixLine=_iconStore->find_iconByLineId(line->get_id());
		if(pixLine) {
			_linePixmap->setPixmap(*pixLine);
		}
	}
	
	if(!_subwayMap)
		return;

	ObjectListIterator routeIt;
	RouteOfLine *tmpRoute=_line->first_routeOfLine(routeIt);
	while(tmpRoute) {
		QString strRoute((const char *)tmpRoute->get_name());
		_routeCombo->insertItem(strRoute);
		if(tmpRoute==routeOfLine)
			indexOfCurrentRoute=index;
		index++;
		tmpRoute=_line->next_routeOfLine(routeIt);
	}
	_routeCombo->setCurrentItem(indexOfCurrentRoute);
	
	selectedRouteOfLine(indexOfCurrentRoute);
}

void LineInfoDialog::set_operationTimes()
{
	const char *daynames[]={
		"Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"
	}; // ohje...
	
	_operationTimeView->clear();
	
	if(!_routeOfLine)
		return;
		
		
	OperationTime& opTime=_routeOfLine->get_optime();

	QListViewItem *lastItem=NULL;
		
	for(int day=0; day<7; day++) {
		QString strDay(daynames[day]);
		QString strHours;
		for(int hour=0; hour<24; hour++) {
			if(opTime.operates_at(static_cast<OperationTime::EDay>(day), hour)) {
				if(hour<10)
					strHours+="0";
				strHours+=QString::number(hour);
				strHours+=" ";
			} else {
				strHours+="   ";
			}
		}
		OperationTimeItem *item=new OperationTimeItem(_operationTimeView, lastItem, strDay, strHours);
		lastItem=item;
	}
}

void LineInfoDialog::selectedRouteOfLine(int index)
{
  if(!_line || !_subwayMap)
  	return;

  _routeOfLineView->clear();
  
  ObjectListIterator routeIt;
  RouteOfLine *routeOfLine=_line->first_routeOfLine(routeIt);
  for(int i=0; i<index; i++) {
  	routeOfLine=_line->next_routeOfLine(routeIt);
  }
  
  if(!routeOfLine)
  	return;

  _routeOfLine=routeOfLine;
  
  LineInfoRouteItem *lastItem=NULL;
  LineInfoRouteItem *item=NULL;
    
  ObjectListIterator it;
  StopAt *stopAt=routeOfLine->first_stop(it);
  while(stopAt) {
	Station *station=stopAt->get_station();
	if(station) {
	  
	  item=new LineInfoRouteItem(_routeOfLineView, lastItem,
	  							(const char *)station->get_name(), 
								stopAt->get_timeToNext(), stopAt->get_timeToPrev(),
								routeOfLine->get_oneway());
	  lastItem=item;

	  ObjectListIterator ctlIt;
	  ChangeToLine *ctl=stopAt->first_changeToLine(ctlIt);
	  while(ctl) {
		Line *line;
		line=_subwayMap->find_lineById(ctl->get_id());
		if(line) {
		  QListViewItem *ctlItem=new QListViewItem(item, (const char *)line->get_name());
		  if(_iconStore) {
			QPixmap *linePixmap=NULL;
			linePixmap=_iconStore->find_iconByLineId(line->get_id());
			if(linePixmap) ctlItem->setPixmap(0, *linePixmap);
		  }
		}
		ctl=stopAt->next_changeToLine(ctlIt);
	  }
	  stopAt=routeOfLine->next_stop(it);
	} // if(station)
  }
  
  set_operationTimes();
}

void LineInfoDialog::cleanup()
{
	_routeOfLineView->clear();
	_linePixmap->clear();
	_lineLabel->clear();
	_routeCombo->clear();
	_operationTimeView->clear();
}

LineInfoRouteItem::LineInfoRouteItem(QListView *parent, QListViewItem *lastItem, const char *strStation, int ttn, int ttp, bool oneway)
	: QListViewItem(parent, lastItem)
{
  QPixmap pixmap(oneway ? oneway_xpm : twoway_xpm);

  setText(0, strStation);
  setPixmap(0, pixmap);
  QString strTtn;
  strTtn.setNum(ttn);
  setText(1, strTtn);
  QString strTtp;
  strTtp.setNum(ttp);
  setText(2, strTtp);

}

LineInfoRouteItem::~LineInfoRouteItem()
{
}

OperationTimeItem::OperationTimeItem(QListView *parent, QListViewItem *lastItem, QString& dayname, QString& operationTimes)
 : QListViewItem(parent, lastItem)
{
	setText(0, dayname);
	setText(1, operationTimes);
}
