/*
#########################################################################
# Subway Map
# ----------
# Copyright (c) 2004 by Dimitri Brukakis
# All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
#########################################################################
*/

#include "LineListDialog.h"
#include "images.h"

LineListDialog::LineListDialog(QWidget *parent, const char *name, bool modal, WFlags fl)
  : LineListDialogBase(parent, name, modal, fl),
    _lineInfoDialog(NULL)
{
	connect((QObject*)_linesList, SIGNAL(doubleClicked(QListViewItem *)), this, SLOT(lineSelected(QListViewItem *)));
	
}

LineListDialog::~LineListDialog()
{
	if(_lineInfoDialog)
		delete _lineInfoDialog;
	_lineInfoDialog=NULL;
}


void LineListDialog::load_subwayMap(SubwayMap *subwayMap)
{
  if(!subwayMap) return;

  _subwayMap=subwayMap;

  // ---- Load lines from subway map ------------------
  ObjectListIterator it;

  LineListItem *lastItem=NULL;
  
  Line *line=_subwayMap->first_line(it);
  while(line) {
	QString strLine((const char *)line->get_name());

	strLine+=": (";

	strLine+=(const char *)line->get_class();
	strLine+=")";

	QPixmap *pixLine=NULL;
	if(_iconStore) {
		pixLine=_iconStore->find_iconByLineId(line->get_id());
	}
	
	LineListItem *item=new LineListItem(_linesList, lastItem, pixLine, strLine, line);
	lastItem=item;
	
	line=_subwayMap->next_line(it);
  }

}

void LineListDialog::clear_lines()
{
  _linesList->clear();
}

void LineListDialog::lineSelected(QListViewItem *item)
{
  LineListItem *lineItem=static_cast<LineListItem *>(item);
  
  if(!lineItem || lineItem->get_line()==NULL) return;

  	
  if(!_lineInfoDialog) {
  	_lineInfoDialog=new LineInfoDialog(this);
  }
  if(!_lineInfoDialog)
  	return; // out of memory
  	
  _lineInfoDialog->set_subwayMap(_subwayMap);
  _lineInfoDialog->set_iconStore(_iconStore);
  _lineInfoDialog->set_lineAndRoute(lineItem->get_line(), NULL);
  _lineInfoDialog->exec();
  _lineInfoDialog->cleanup();
}


LineListItem::LineListItem(QListView *parent, QListViewItem *lastItem, QPixmap *pixmap, const char *name, Line *line)
  : QListViewItem(parent, lastItem), _line(line)
{
	if(pixmap) {
		setPixmap(0, *pixmap);
	}
	setText(1, QString(name));
}

LineListItem::~LineListItem()
{
}

