/*
#########################################################################
# Subway Map
# ----------
# Copyright (c) 2004 by Dimitri Brukakis
# All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
#########################################################################
*/

#ifndef __ROUTELISTVIEW_H__
#define __ROUTELISTVIEW_H__


#include <subwaymap/RouteFinder.h>

#include <qlistview.h>
#include <qpixmap.h>


#include "UBahnIconStore.h"

class RouteListItem;
class RouteInfoItem;

class RouteListView : public QListView
{
  Q_OBJECT;

 public:
  RouteListView(QWidget *parent, const char *name=0);
  virtual ~RouteListView();

  void cleanup();
  void addRoute(Route *route);

  void set_showPassedStations(bool show) { _showPassedStations=show; }

  void setLastItem(QListViewItem *lastItem) { _lastItem=lastItem; }
  QListViewItem *getLastItem() { return _lastItem; }

  void set_iconStore(UBahnIconStore *iconStore) { _iconStore=iconStore; }

 protected:
  QListViewItem *_lastItem;

  friend class RouteItemStation;
  friend class RouteItemInfo;
  friend class RouteItemLine;
  friend class RouteItemSeparator;
  friend class RouteItemByFeet;
  friend class RouteItemPassedStation;

  UBahnIconStore *_iconStore;

  bool _showPassedStations;
};

class RouteItem : public QListViewItem
{
 public:
 	enum ERouteItemType {
 		typeUnknown=0,
 		typeStation,
 		typeInfo,
 		typeLine,
 		typeByFeet,
 		typePassedStation,
 		typeSeparator
 	};	
 	virtual ~RouteItem() {}
 	
 	ERouteItemType get_itemType() { return _itemType; }

	Station *get_fromStation() { return _fromStation; }
	Station *get_toStation() { return _toStation; }
	Station *get_station() { return _fromStation; }
	
	Line *get_line() { return _line; }
	RouteOfLine *get_routeOfUsedLine() { return _routeOfUsedLine; }
	  
	void set_fromStation(Station *station) { _fromStation=station; }
	void set_toStation(Station *station) { _toStation=station; }
	void set_station(Station *station) { _fromStation=station; }
	void set_line(Line *line) { _line=line; }
	void set_routeOfUsedLine(RouteOfLine *routeOfLine) { _routeOfUsedLine=routeOfLine; }
 	
 protected:
   RouteItem(RouteListView *parent, QListViewItem *lastitem)
   	: QListViewItem(parent, parent->getLastItem()),
   	  _itemType(typeUnknown),
   	  _fromStation(NULL), _toStation(NULL), _line(NULL), _routeOfUsedLine(NULL) {}

   ERouteItemType _itemType;
   Station *_fromStation;
   Station *_toStation;
   Line *_line;
   RouteOfLine *_routeOfUsedLine;

};

class RouteItemStation : public RouteItem
{
 public:
 
   enum ERouteItemStationType {
 	 typeNormal=0,
 	 typeStart,
 	 typeDestination
   };
 
 public:
  RouteItemStation(RouteListView *parent, QPixmap& pixmap, const char *stationName, unsigned int timeToChange);
  virtual ~RouteItemStation(){}
  
  ERouteItemStationType get_stationType() { return _stationType; }
  void set_stationType(ERouteItemStationType type) { _stationType=type; }
 
 protected:
  ERouteItemStationType _stationType;
};

class RouteItemInfo : public RouteItem
{
 public:
  RouteItemInfo(RouteListView *parent, QPixmap& pixmap, const char *strInfo);
  virtual ~RouteItemInfo() {}
};

class RouteItemLine : public RouteItem
{
 public:
  RouteItemLine(RouteListView *parent, QPixmap& pix0, QPixmap *linePixmap, const char *line, const char *direction,
				int timeToGo);
  virtual ~RouteItemLine() {}


 protected:
};

class RouteItemByFeet : public RouteItem
{
 public:
  RouteItemByFeet(RouteListView *parent, QPixmap& pix0, const char *direction, int timeToGo);
  virtual ~RouteItemByFeet() {}

  
 protected:
};

class RouteItemPassedStation : public RouteItem
{
 public:
  RouteItemPassedStation(RouteListView *parent, QPixmap& pixmap, const char *stationName);
  virtual ~RouteItemPassedStation() {}

  
 protected:
};

class RouteItemSeparator : public RouteItem
{
 public:
  RouteItemSeparator(RouteListView *parent);
  virtual ~RouteItemSeparator() {}
};

#endif
