/*
#########################################################################
# Subway Map
# ----------
# Copyright (c) 2004 by Dimitri Brukakis
# All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
#########################################################################
*/

#include "StationListDialog.h"

static const char* ulogo_data[] = { 
"15 15 10 1",
"a c #055794",
"c c #3277a8",
"# c #3e3d70",
"b c #6a9dc1",
". c #7d2c4d",
"g c #91bcd3",
"h c #bfd8e6",
"d c #e1ecef",
"f c #fdedee",
"e c #ffffff",
".##.#######.##.",
"#abbcaaaaacbba#",
"#cdebaaaaabefc#",
"#adegaaaaabedc#",
"#cdeb#aaaabedc#",
"#adebaaaa#bedc#",
"#cdebaaaaabedc#",
"#adegaaaaabedc#",
"#cdeb#aaaabedc#",
"#adegaaaa#bedc#",
"#ahedc##aadeha#",
"#abeedbcbheeba#",
"#aageeeeeeegaa#",
"#aaachhddhbaaaa",
"#aa#a#cca#a#aa#"};



StationListDialog::StationListDialog(QWidget *parent, const char *name, bool modal, WFlags fl)
  : QDialog(parent, name, modal, fl), _iconStore(NULL)
{
  if(!name)
    setName( "StationListDialog" );

  int width=240;
  int height=290;

  if(parent) {
	width=parent->width();
	height=parent->height();
  }
  _parent=parent;

  resize( width, height );

  setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)4, (QSizePolicy::SizeType)4, sizePolicy().hasHeightForWidth() ) );
  //setMaximumSize( QSize( width, height ) );
  setCaption( tr( "Stations" ) );

  _stationListView = new QListView( this, "_stationListView" );
  _stationListView->addColumn( "Stations" );
  _stationListView->setGeometry( QRect( 0, 0, 240, 240 ) );
  _stationListView->setResizePolicy( QScrollView::Manual );
  _stationListView->setRootIsDecorated( TRUE );
}

StationListDialog::~StationListDialog()
{
}

void StationListDialog::resizeIt()
{
  if(_parent) {
	resize(_parent->width(), _parent->height());
	_stationListView->resize(_parent->width(), _parent->height()-1);
  }
}

int StationListDialog::load_subwayMap(SubwayMap *subwayMap)
{
  if(!subwayMap) return -1;
    
  _subwayMap=subwayMap;


  QString str;
  ObjectListIterator it;
  Station *station=_subwayMap->first_station(it);
  while(station) {
	str=(const char *)station->get_name();
	QListViewItem *listItem=new QListViewItem(_stationListView, str);

	ObjectListIterator slIt;
	ServingLine *servingLine=station->first_servingLine(slIt);
	while(servingLine) {
	  Line *line=_subwayMap->find_lineById(servingLine->get_id());
	  if(line)
		str=(const char *)line->get_name();
	  else {
		str="no line for sl=";
		str+=(const char *)servingLine->get_id();
	  }

	  QListViewItem *lineItem=new QListViewItem(listItem, NULL, str);

	  if(_iconStore) {
		QPixmap *linePixmap=NULL;
		linePixmap=_iconStore->find_iconByLineId(servingLine->get_id());
		if(linePixmap) lineItem->setPixmap(0, *linePixmap);
	  }
	  servingLine=station->next_servingLine(slIt);
	}
	station=_subwayMap->next_station(it);
  }
  return 0;
}

int StationListDialog::clear_stations()
{
  _stationListView->clear();
}
