/*
#########################################################################
# Subway Map
# ----------
# Copyright (c) 2004 by Dimitri Brukakis
# All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
#########################################################################
*/

#include <iostream.h>
#include <qmessagebox.h>
#include <qdatetime.h>

#include "UBahnNavMain.h"

#include "OpenMapDialog.h"
#include "UBahnDataFile.h"
#include "OpTimeDialog.h"

#include "images.h"


UBahnNavMain::UBahnNavMain(QWidget *parent, const char *name, WFlags fl)
  : QMainWindow(parent, name, fl),
	_loaded(0),
	_stationListDialog(NULL), _lineListDialog(NULL),
	_aboutDialog(NULL), _mapInfoDialog(NULL),
	_opTimeDialog(NULL), _stationInfoDialog(NULL),
	_lineInfoDialog(NULL),
	_config(g_appDef_UserConfig),
	_useRealTime(true),
	_selectedRouteItem(NULL)
{

  // --- Subway map/route finder debug level
  Debug::set_max_level(Debug::level_silent);

  // ---- Menubar and menues -----------
  _menuBar=NULL;
  _menuToolBar=NULL;
  _fileMenu=NULL;
  _viewMenu=NULL;
  _settingsMenu=NULL;
  _helpMenu=NULL;
  _findRoutePixmap=NULL;
  _toolbar=NULL;
  _openMapButton=NULL;
  _findRouteButton=NULL;
  _returnButton=NULL;
  _timeButton=NULL;
  _statusBar=NULL;
  _statusLabel=NULL;
  _entryToolBar=NULL;
  _entryFrame=NULL;
  _layout1=NULL;
  _startComboBox=NULL;
  _destComboBox=NULL;
  _startLabel=NULL;
  _destLabel=NULL;
  _routeListView=NULL;


  // Create menu bar
  buildMenu();
  // Create main view
  buildMainView();


  // setToolBarsMovable(false);

  int rc;
  rc=chdir(g_appDef_MapDir);
  if(rc<0) { // cannot change to map directory
	QString errStr="Cannot change to dir\n";
	errStr+=g_appDef_MapDir;
	QMessageBox::critical(this, "UBahn Navigator",
						  errStr,
						  "Close", 0, 0, 0, 1);
	return;
  }

  // --- Read saved configuration -------------
  if(_config.read()==true) {
	//cout << "_lastDataFile=" << _config._lastDataFile << endl;
	if(_config._lastDataFile!="none") {
	  rc=loadUBahnDataFile(_config._lastDataFile);
	  if(rc==0) {
		_startComboBox->setCurrentItem(_config._lastStartStation.toInt());
		_destComboBox->setCurrentItem(_config._lastDestStation.toInt());
	  }
	} else {
	  setMapLoaded(false);
	}
	if(_config._lastShowPassedStations=="true") {
	  _viewMenu->setItemChecked(menuId_ShowPassedStations, true);
	}
  }
  whatTimeIsNow();
  
}


UBahnNavMain::~UBahnNavMain()
{
  // save configuration
  cleanupSubwayMap();
  _config.write();
}

void UBahnNavMain::buildMenu()
{
  // ---------- Main menu -------------------------------------
  _menuToolBar=new QToolBar(this);
  _menuToolBar->setHorizontalStretchable(true);
  _menuToolBar->setVerticalStretchable(false);
  _menuToolBar->setOrientation(Horizontal);

  _fileMenu=new QPopupMenu(_menuToolBar);
  _fileMenu->insertItem("&Open", this, SLOT(openMap()), CTRL+Key_O);
  _fileMenu->insertItem("&Close", this, SLOT(closeMap()), CTRL+Key_C, (int)menuId_FileClose);
  _fileMenu->insertSeparator();
  _fileMenu->insertItem("E&xit", this, SLOT(exitApp()), CTRL+Key_X);

  _viewMenu=new QPopupMenu(_menuToolBar);
  _viewMenu->insertItem("Map &Info", this, SLOT(mapInfo()), CTRL+Key_I, (int)menuId_ShowInfo);
  _viewMenu->insertItem("&Lines", this, SLOT(showLines()), CTRL+Key_L, (int)menuId_ShowLines);
  _viewMenu->insertItem("&Stations", this, SLOT(showStations()), CTRL+Key_S, (int)menuId_ShowStations);
  _viewMenu->insertSeparator();
  _viewMenu->insertItem("Show &passed stations", this, SLOT(showPassedStations()), 0, 
						(int)menuId_ShowPassedStations);

  _viewMenu->setItemEnabled(menuId_ShowAlternatives, false);


  _helpMenu=new QPopupMenu(_menuToolBar);
  _helpMenu->insertItem("&About", this, SLOT(helpAbout()));

  _menuBar=new QMenuBar(_menuToolBar);
  _menuBar->insertItem("&File", _fileMenu);
  _menuBar->insertItem("&View", _viewMenu);
  _menuBar->insertItem("&Help", _helpMenu);

  // ---------- Toolbar -------------------------------------

  QPixmap findRoutePixmap((const char **)nav2_xpm);
  QPixmap returnPixmap((const char **)return_xpm);
  QPixmap openMapPixmap((const char **)fileopen_xpm);
  QPixmap setTimePixmap((const char **)settime_xpm);
  QPixmap connectionsPixmap((const char **)connections_xpm);

  _toolbar=new QToolBar(this);
  _toolbar->setHorizontalStretchable(true);
  _toolbar->setOrientation(Horizontal);

  _openMapButton=new QToolButton(openMapPixmap, "Open Map", "Open U-Bahn map", this, SLOT(openMap()), _toolbar);
  _findRouteButton=new QToolButton(findRoutePixmap, "Find Route", "Find route", this, SLOT(findRoute()), _toolbar);
  _returnButton=new QToolButton(returnPixmap, "Return", "Return", this, SLOT(switchStations()), _toolbar);
  _connectionsButton=new QToolButton(connectionsPixmap, "Connections", "Connections", this, SLOT(showConnections()), _toolbar);
  _timeButton=new QToolButton(setTimePixmap, "Set time", "Set time", this, SLOT(setTime()), _toolbar);

  _lineListDialog=new LineListDialog(this);
  _stationListDialog=new StationListDialog(this);
  _altConnectionDialog=new AltConnectionDialog(this);
  _stationListDialog->set_iconStore(&_iconStore);
  _lineListDialog->set_iconStore(&_iconStore);
  _altConnectionDialog->set_iconStore(&_iconStore);

}

void UBahnNavMain::buildMainView()
{
  QPixmap ulogoPixmap((const char**)ulogo_xpm);

  // setCaption(g_appInfo_Name);
  displayMapInCaption(NULL);

  // --------- Build entry tool bar ---------------------------
  _entryToolBar=new QToolBar(this);
  _entryToolBar->setHorizontalStretchable(true);
  _entryToolBar->setVerticalStretchable(false);

  _entryFrame=new QFrame(_entryToolBar, "entryFrame");
  _entryFrame->setGeometry(QRect(0, 0, 240, 75));
  _entryFrame->setFrameStyle(QFrame::Box);
  _entryFrame->setLineWidth(1);
  _entryFrame->setFrameShape( QFrame::StyledPanel );
  _entryFrame->setFrameShadow( QFrame::Raised );

  _layout1=new QGridLayout(_entryFrame, 2, 2);
  _layout1->setSpacing(1);
  _layout1->setMargin(1);


  _startLabel=new QLabel(_entryFrame, "startLabel");
  _startLabel->setSizePolicy(QSizePolicy((QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, 
										 _startLabel->sizePolicy().hasHeightForWidth()));
  _startLabel->setMaximumSize(QSize( 20, 20 ));
  _startLabel->setLineWidth(0);
  _startLabel->setPixmap(ulogoPixmap);
  _startLabel->setScaledContents(TRUE);
  
  _startComboBox=new QComboBox(_entryFrame, "startComboBox");
  _startComboBox->setEditable(false);
  _startComboBox->setAutoCompletion(true);
							  
  _destLabel=new QLabel(_entryFrame, "destLabel");
  _destLabel->setSizePolicy(QSizePolicy((QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, 
										_destLabel->sizePolicy().hasHeightForWidth()));
  _destLabel->setMaximumSize(QSize( 20, 20 ));
  _destLabel->setLineWidth(0);
  _destLabel->setPixmap(ulogoPixmap);
  _destLabel->setScaledContents(TRUE);

  _destComboBox=new QComboBox(_entryFrame, "destComboBox");
  _destComboBox->setEditable(false);
  _destComboBox->setAutoCompletion(true);


  _layout1->addWidget(_startLabel, 0, 0);
  _layout1->addWidget(_startComboBox, 0, 1);
  _layout1->addWidget(_destLabel, 1, 0);
  _layout1->addWidget(_destComboBox, 1, 1);


  // ------------ Status Bar -------------------------------------
  _statusBar=new QStatusBar(this);
  _statusBar->setSizeGripEnabled(false);
  displayStatus("...", 0);

  // -------------------------------------------------------------

  _routeListView=new RouteListView(this, "routeListView");
  _routeListView->set_iconStore(&_iconStore);

  setCentralWidget(_routeListView);

 connect((QObject*)_routeListView, SIGNAL(doubleClicked(QListViewItem*)), 
 			this, SLOT(routeListDblClicked(QListViewItem*)));
 connect((QObject*)_routeListView, SIGNAL(clicked(QListViewItem*)), 
 			this, SLOT(routeListClicked(QListViewItem*)));

}

/*	======================================================
	Slots
	====================================================== */

void UBahnNavMain::openMap()
{
  OpenMapDialog openMapDialog;
  int rc;
  QString errStr;

  rc=chdir(g_appDef_MapDir);
  if(rc<0) { // cannot change to map directory
	errStr="Cannot change to dir\n";
	errStr+=g_appDef_MapDir;
	QMessageBox::critical(this, "UBahn Navigator",
						  errStr,
						  "Close", 0, 0, 0, 1);
	return;
  }

  openMapDialog.readDirectory(g_appDef_MapDir);

  if((rc=openMapDialog.exec())==1) {
	closeMap();
	loadUBahnDataFile(openMapDialog.get_fileName());
  }

}

void UBahnNavMain::closeMap()
{
  cleanupSubwayMap();
  cleanupStationComboLists();
  _routeListView->cleanup();
  setMapLoaded(false);
  _config._lastDataFile="none";

  displayMapInCaption(NULL);
}

void UBahnNavMain::exitApp()
{
  close();
}

void UBahnNavMain::mapInfo()
{
  if(!_mapInfoDialog) {
	_mapInfoDialog=new MapInfoDialog;
  }
  _mapInfoDialog->load_subwayMap(&_subwayMap);
  _mapInfoDialog->exec();
}

void UBahnNavMain::showLines()
{
  _lineListDialog->load_subwayMap(&_subwayMap);
  _lineListDialog->exec();
  _lineListDialog->clear_lines();
}

void UBahnNavMain::showStations()
{
  _stationListDialog->load_subwayMap(&_subwayMap);
  _stationListDialog->resizeIt();
  _stationListDialog->exec();
  _stationListDialog->clear_stations();
}

void UBahnNavMain::showPassedStations()
{
  _viewMenu->setItemChecked(menuId_ShowPassedStations, 
							!_viewMenu->isItemChecked(menuId_ShowPassedStations));
  _config._lastShowPassedStations=(_viewMenu->isItemChecked(menuId_ShowPassedStations))?"true":"false";

  finishRouteFinder();
}

void UBahnNavMain::helpAbout()
{
  if(!_aboutDialog) {
	_aboutDialog=new AboutDialog;
  }
  _aboutDialog->exec();
}
void UBahnNavMain::findRoute()
{
  // --- start route finder ----

  startRouteFinder();
}

void UBahnNavMain::switchStations()
{
  int startItem=_startComboBox->currentItem();
  int destItem=_destComboBox->currentItem();

  _startComboBox->setCurrentItem(destItem);
  _destComboBox->setCurrentItem(startItem);
}

void UBahnNavMain::setTime()
{
  int rc;
  
  if(!_opTimeDialog) {
	_opTimeDialog=new OpTimeDialog;
  }
  _opTimeDialog->set_dayAndTime(static_cast<unsigned int>(_actOperationDay), _actOperationHour);
  if((rc=_opTimeDialog->exec())==1) {
  	unsigned int optime;
  	_opTimeDialog->get_dayAndTime(&optime, &_actOperationHour);
  	_actOperationDay=static_cast<OperationTime::EDay>(optime);
  	_useRealTime=!_opTimeDialog->has_selected();
  }
}

void UBahnNavMain::routeListDblClicked(QListViewItem *item)
{
	if(item==NULL)
		return;
		
	RouteItem *routeItem=static_cast<RouteItem *>(item);
	Station *fromStation=routeItem->get_fromStation();
	Station *toStation=routeItem->get_toStation();
	Line *line=routeItem->get_line();
	RouteOfLine *routeOfLine=routeItem->get_routeOfUsedLine();
	
	printf("doubleClicked: RouteItem, type=%d\n", routeItem->get_itemType());
	printf("from %s to %s line %s\n",
		(fromStation==NULL ? "" :(const char*)fromStation->get_name()),
		(toStation==NULL   ? "" :(const char*)toStation->get_name()),
		(line==NULL        ? "" :(const char*)line->get_name())); 

	if(routeItem->get_itemType()==RouteItem::typeStation ||
	   	routeItem->get_itemType()==RouteItem::typePassedStation) {
	   	RouteItemStation *tmp=static_cast<RouteItemStation*>(routeItem);
	   	if(tmp->get_stationType()!=RouteItemStation::typeDestination)
	   		showStationInfo(fromStation);
	   	else
	   		showStationInfo(toStation);
	}	
	
	if(routeItem->get_itemType()==RouteItem::typeLine) {
		showLineInfo(line, routeOfLine);
	}
}


void UBahnNavMain::routeListClicked(QListViewItem *item)
{
	_selectedRouteItem=NULL;
	
	if(item==NULL)
		return;
		
	RouteItem *routeItem=static_cast<RouteItem *>(item);

	printf("clicked: RouteItem, type=%d\n", routeItem->get_itemType());	

	if(routeItem->get_itemType()==RouteItem::typeStation) {
		RouteItemStation *tmp=static_cast<RouteItemStation*>(routeItem);
		if(tmp->get_stationType()!=RouteItemStation::typeDestination)
			_selectedRouteItem=routeItem;
	}
	if(routeItem->get_itemType()==RouteItem::typeLine) {
		_selectedRouteItem=routeItem;
	}
	
	if(_selectedRouteItem) {
		_connectionsButton->setEnabled(true);
	} else {
		_connectionsButton->setEnabled(false);
	}
}

void UBahnNavMain::showConnections()
{
	if(!_selectedRouteItem){
		_viewMenu->setItemEnabled(menuId_ShowAlternatives, false);
		return;
	}
	if(!_altConnectionDialog)
		return;
		
	_altConnectionDialog->set_startStation(_selectedRouteItem->get_fromStation());
	_altConnectionDialog->set_destStation(_selectedRouteItem->get_toStation());
	_altConnectionDialog->set_subwayMap(&_subwayMap);
	_altConnectionDialog->start_findConnections();
	_altConnectionDialog->exec();
	_altConnectionDialog->cleanup();
}


void UBahnNavMain::setMapLoaded(bool loaded)
{
  if(loaded==true) {
	_loaded=1;

	// ---- enable UI elements -----
	_fileMenu->setItemEnabled(menuId_FileClose, true);
	_viewMenu->setItemEnabled(menuId_ShowInfo, true);
	_viewMenu->setItemEnabled(menuId_ShowLines, true);
	_viewMenu->setItemEnabled(menuId_ShowStations, true);
	
	_findRouteButton->setEnabled(true);
	_returnButton->setEnabled(true);
	_connectionsButton->setEnabled(false);

  } else {
	_loaded=0;

	// ---- disable UI elements ----

	_fileMenu->setItemEnabled(menuId_FileClose, false);
	_viewMenu->setItemEnabled(menuId_ShowInfo, false);
	_viewMenu->setItemEnabled(menuId_ShowLines, false);
	_viewMenu->setItemEnabled(menuId_ShowStations, false);

	_findRouteButton->setEnabled(false);
	_returnButton->setEnabled(false);
	_connectionsButton->setEnabled(false);
  }
}


void UBahnNavMain::displayStatus(const char *what, int duration, const char *param1, const char *param2)
{
  QString statusString=what;

  if(param1) statusString+=param1;
  if(param2) statusString+=param2;

  if(duration)
	_statusBar->message(statusString, duration*1000);
  else
	_statusBar->message(statusString);
}

void UBahnNavMain::displayMapInCaption(const char *mapname)
{
  QString caption=g_appInfo_Name;

  if(mapname) {	// Display loaded map in mainwindow caption
	caption+=":";
	caption+=mapname;
  } else {		// no map? Display application version instead
	caption+=" ";
	caption+=g_appInfo_Version;
  }	
  setCaption(caption);

}

void UBahnNavMain::showStationInfo(Station *station)
{
	if(!_stationInfoDialog) {
		_stationInfoDialog=new StationInfoDialog;
	}
	if(!_stationInfoDialog)
		return; // out of memory
		
	_stationInfoDialog->set_subwayMap(&_subwayMap);
	_stationInfoDialog->set_iconStore(&_iconStore);
	_stationInfoDialog->set_station(station);
	_stationInfoDialog->exec();	
	_stationInfoDialog->cleanup();
}

void UBahnNavMain::showLineInfo(Line *line, RouteOfLine *routeOfLine)
{
	if(!_lineInfoDialog) {
		_lineInfoDialog=new LineInfoDialog;
	}
	
	if(!_lineInfoDialog)
		return; // out of memory
		
	_lineInfoDialog->set_subwayMap(&_subwayMap);
	_lineInfoDialog->set_iconStore(&_iconStore);
	_lineInfoDialog->set_lineAndRoute(line, routeOfLine);
	_lineInfoDialog->exec();
	_lineInfoDialog->cleanup();
}


/*	======================================================
	UBahnData and Subway map methods
	====================================================== */
int UBahnNavMain::loadUBahnDataFile(QString& filename)
{
  QString errStr;
  int rc;
  UBahnDataFile dataFile(filename);

  if(dataFile.read()==false) {
	errStr="Cannot open file\n";
	errStr+=filename;
	QMessageBox::critical(this, "UBahn Navigator",
							errStr,
							"Close", 0, 0, 0, 1);
	return -1;
  }

  if((rc=chdir(dataFile._mapDir))) {
	errStr="Map dir not found\n";
	errStr+=dataFile._mapDir;
	QMessageBox::critical(this, "UBahn Navigator",
							errStr,
							"Close", 0, 0, 0, 1);
	return -1;
  }

  rc=loadSubwayMap((const char *)dataFile._subwayMapFile);
  if(rc!=0) {
	return -1;
  }

  rc=loadIconStore((const char *)dataFile._iconFile);
  if(rc!=0) {
	return -1;
  }

  _config._lastDataFile=filename;
  return 0;
}

int UBahnNavMain::loadSubwayMap(const char *filename)
{
  int rc;
  SMXMLSubwayMapParser_Qt parser;
  String fileName=filename;

  _subwayMap.cleanup();

  rc=parser.parse(filename, &_subwayMap);
  if(rc==smeOK) {
	// QMessageBox::information(this, "U-Bahn Navigator",
	//					(const char *)_subwayMap.get_name(),
	//					"Ok", 0, 0, 0, 1);

	setMapLoaded(true);

	buildStationComboLists();
	displayStatus("Loaded map ", 3, (const char *)_subwayMap.get_name());
	displayMapInCaption((const char *)_subwayMap.get_name());
	_routeFinder.use_subwayMap(&_subwayMap);
	return 0;
  }


  if(rc==smeGOT_XML_EXCEPTION) {
	XMLException *e=(XMLException *)parser.get_exception();
	QString errStr;
	errStr.sprintf("XML (map): %s\nTag: %s\nline %d, col %d\n",
				   (const char *)e->get_errstr(),
				   (const char *)e->get_xmlTag(),
				   e->get_line(), e->get_column());
	QMessageBox::critical(this, "UBahn Navigator",
						  errStr,
						  "Close", 0, 0, 0, 1);
	parser.clean_exception();
	return rc;
  }
  if(rc==smeGOT_FILE_EXCEPTION) {
	FileException *e=(FileException *)parser.get_exception();
	QString errStr;
	errStr.sprintf("File (map): %s\n%s",
				   (const char *)e->get_errstr(),
				   (const char *)e->get_filename());
	QMessageBox::critical(this, "U-Bahn Navigator",
						  errStr,
						  "Close", 0, 0, 0, 1);
	parser.clean_exception();
	return rc;
  }
  ExceptionBase *e=(ExceptionBase *)parser.get_exception();
  if(e) {
	QMessageBox::critical(this, "U-Bahn Navigator",
						(const char *)e->get_errstr(),
						"Close", 0, 0, 0, 1);
  	parser.clean_exception();
  }
  
  return rc;
}

int UBahnNavMain::loadIconStore(const char *filename)
{
  int rc;

  XMLIconFileParser_Qt parser;

  _iconStore.cleanup();

  rc=parser.parse(filename, &_iconStore);
  if(rc==smeOK) {
	// ok, what else to do?
	return 0;
  }

  if(rc==smeGOT_XML_EXCEPTION) {
	XMLException *e=(XMLException *)parser.get_exception();
	QString errStr;
	errStr.sprintf("XML (icon): %s\nTag: %s\nline %d, col %d\n",
				   (const char *)e->get_errstr(),
				   (const char *)e->get_xmlTag(),
				   e->get_line(), e->get_column());
	QMessageBox::critical(this, "UBahn Navigator",
						  errStr,
						  "Close", 0, 0, 0, 1);
	parser.clean_exception();
	return rc;
  }

  if(rc==smeGOT_FILE_EXCEPTION) {
	FileException *e=(FileException *)parser.get_exception();
	QString errStr;
	errStr.sprintf("(icon): %s\n%s",
				   (const char *)e->get_errstr(),
				   (const char *)e->get_filename());
	QMessageBox::critical(this, "UBahn Navigator",
						  errStr,
						  "Close", 0, 0, 0, 1);
	parser.clean_exception();
	return rc;
  }
  ExceptionBase *e=(ExceptionBase *)parser.get_exception();
  QMessageBox::critical(this, "UBahn Navigator",
						(const char *)e->get_errstr(),
						"Close", 0, 0, 0, 1);
  parser.clean_exception();
  return rc;
}


/*	======================================================
	route finder methods
	====================================================== */

int UBahnNavMain::startRouteFinder()
{
  String startStation;
  String destStation;
  int rc;


  // --- Read stations from combo box
  startStation=(const char *)_startComboBox->currentText();
  destStation=(const char *)_destComboBox->currentText();

  // --- Store station information in config file
  _config._lastStartStation.sprintf("%d", _startComboBox->currentItem());
  _config._lastDestStation.sprintf("%d", _destComboBox->currentItem());

  if(startStation==destStation) {
	QMessageBox::critical(this, g_appInfo_Name,
						  "Start eq. destination",
						  "Ok", 0, 0, 0, 1);
	return 0;
  }

  // --- put stations into route finder

  _routeFinder.set_startStationByName(startStation);
  _routeFinder.set_destStationByName(destStation);
  
  // --- set acutal or selected time
  if(_useRealTime) {
  	printf("_useRealTime\n");
  	whatTimeIsNow();
  }
  _routeFinder.set_operationTime(_actOperationDay, _actOperationHour);


  // --- remove previously found routes
  _routeFinder.cleanup_routesFound();

  _connectionsButton->setEnabled(false);
  _routeListView->cleanup();

  displayStatus("find route...", 0);

  // --- start route finder

  rc=_routeFinder.findRoute();
  if(rc==smeOK) {
	return finishRouteFinder();
  }

  if(rc==smeGOT_LINE_NOT_FOUND_EXCEPTION) {
    SubwayMapLineNotFound *e=(SubwayMapLineNotFound *)_routeFinder.get_exception();
	QString errStr;
	errStr.sprintf("Internal: %s", (const char *)e->get_errstr());
	QMessageBox::critical(this, g_appInfo_Name,
						  errStr,
						  "Ok", 0, 0, 0, 1);
	displayStatus("line not found", 3);
	_routeFinder.clean_exception();
	return 0;
  }
  if(rc==smeGOT_SUBWAYMAP_EXCEPTION) {
    SubwayMapException *e=(SubwayMapException *)_routeFinder.get_exception();
	QMessageBox::critical(this, g_appInfo_Name,
						  (const char *)e->get_errstr(),
						  "Ok", 0, 0, 0, 1);
	displayStatus((const char *)e->get_errstr(), 3);
	_routeFinder.clean_exception();
	return 0;
  }
  
  ExceptionBase *e=_routeFinder.get_exception();
  if(e) {
	  QMessageBox::critical(this, g_appInfo_Name,
						  (const char *)e->get_errstr(),
						  "Ok", 0, 0, 0, 1);
	  displayStatus((const char *)e->get_errstr(), 3);
	  _routeFinder.clean_exception();
	  
  } else {
  	  QString errStr;
  	  errStr.sprintf("Could not find a route");
  	  QMessageBox::critical(this, g_appInfo_Name,
						  errStr,
						  "Ok", 0, 0, 0, 1);
	  displayStatus(errStr, 3);
  }

  return 0;
}

int UBahnNavMain::finishRouteFinder()
{
  //cout << "finishRouteFinder()" << endl;
  displayStatus("done", 3);

  _routeListView->cleanup();

  // --- insert routes found to RouteView


  bool show=_viewMenu->isItemChecked(menuId_ShowPassedStations);
  _routeListView->set_showPassedStations(show);

  ObjectListIterator it;
  Route *route=_routeFinder.first_routeFound(it);
  while(route) {
	_routeListView->addRoute(route);
	route=_routeFinder.next_routeFound(it);
  }

  return 0;
}


int UBahnNavMain::cleanupSubwayMap()
{
  _subwayMap.cleanup();
  _routeFinder.cleanup();
  _iconStore.cleanup();
  return 0;
}


void UBahnNavMain::buildStationComboLists()
{

  _subwayMap.sort_stations();

  ObjectListIterator it;
  Station *station=_subwayMap.first_station(it);
  while(station) {
	_startComboBox->insertItem(QString((const char *)station->get_name()));
	_destComboBox->insertItem(QString((const char *)station->get_name()));
	station=_subwayMap.next_station(it);
  }
}

void UBahnNavMain::cleanupStationComboLists()
{
  _startComboBox->clear();
  _destComboBox->clear();
}

void UBahnNavMain::whatTimeIsNow()
{
	int dayOfWeek=QDate::currentDate().dayOfWeek();
	int hourOfDay=QTime::currentTime().hour();
	
	_actOperationHour=static_cast<unsigned int>(hourOfDay);
	_actOperationDay=static_cast<OperationTime::EDay>(dayOfWeek-1);		
}


