/*
#########################################################################
# Subway Map
# ----------
# Copyright (c) 2004 by Dimitri Brukakis
# All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
#########################################################################
*/

#ifndef __UBAHNNAVMAIN_H__
#define __UBAHNNAVMAIN_H__

// #include "ubahnnavmainbase.h"


#include <qmenubar.h>
#include <qmainwindow.h>
#include <qtoolbar.h>
#include <qcombobox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qtoolbutton.h>
#include <qstatusbar.h>
#include <qhbuttongroup.h>
#include <qpushbutton.h>

#include <subwaymap/SubwayMap.h>
#include <subwaymap/SMXMLSubwayMapParser_Qt.h>
#include <subwaymap/RouteFinder.h>

#include "StationListDialog.h"
#include "LineListDialog.h"
#include "AboutDialog.h"
#include "MapInfoDialog.h"
#include "RouteListView.h"
#include "OpTimeDialog.h"
#include "StationInfoDialog.h"
#include "LineInfoDialog.h"
#include "AltConnectionDialog.h"

#include "SimpleConfig.h"

#include "XMLIconFileParser.h"
#include "UBahnIconStore.h"

#include "ubahnnavdef.h"


class UBahnNavMain : public QMainWindow // UBahnNavMainBase
{
  Q_OBJECT;

 public:
  UBahnNavMain( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
  virtual ~UBahnNavMain();

  void displayStatus(const char *what, int duration, const char *param1=NULL, const char *param2=NULL);
  void displayMapInCaption(const char *mapname);

 protected:
  // ==== UI Elements ================================
  void buildMenu();
  void buildMainView();

  enum eMenuId {
	menuId_FileClose=1,		// File Menu
	menuId_ShowInfo=1,		// Map Menu
	menuId_ShowLines=2,
	menuId_ShowStations=3,
	menuId_ShowPassedStations=4,
	menuId_ShowAlternatives=5
  };

  // ---- Menubar and menues -----------
  QMenuBar *_menuBar;
  QToolBar *_menuToolBar;
  QPopupMenu *_fileMenu;
  QPopupMenu *_viewMenu;
  QPopupMenu *_settingsMenu;
  QPopupMenu *_helpMenu;
  QPixmap *_findRoutePixmap;


  // ---- Toolbar ----------------------
  QToolBar *_toolbar;
  QToolButton *_openMapButton;
  QToolButton *_findRouteButton;
  QToolButton *_returnButton;
  QToolButton *_timeButton;
  QToolButton *_connectionsButton;


  QStatusBar *_statusBar;
  QLabel *_statusLabel;

  // ---- Station entry toolbox --------
  QToolBar *_entryToolBar;
  QFrame *_entryFrame;
  QGridLayout *_layout1;
  QComboBox *_startComboBox;
  QComboBox *_destComboBox;
  QLabel *_startLabel;
  QLabel *_destLabel;

  // ---- Listview for route output ----
  RouteListView *_routeListView;

  // ---- UI Information ----------------
  void setMapLoaded(bool loaded=true);
  int _loaded;

 protected:
  // ==== Other dialogs ===============================

  StationListDialog *_stationListDialog;
  LineListDialog *_lineListDialog;
  AboutDialog *_aboutDialog;
  MapInfoDialog *_mapInfoDialog;
  OpTimeDialog *_opTimeDialog;
  StationInfoDialog *_stationInfoDialog;
  LineInfoDialog *_lineInfoDialog;
  AltConnectionDialog *_altConnectionDialog;

 protected:
  // ==== Configuration ===============================
  SimpleConfig _config;

 protected:
  // ==================================================
  // Subway Map Library components
  // ==================================================
  int loadUBahnDataFile(QString& filename);
  int loadSubwayMap(const char *filename);
  int loadIconStore(const char *filename);
  int cleanupSubwayMap();

  int startRouteFinder();
  int finishRouteFinder();

  SubwayMap _subwayMap;
  RouteFinder _routeFinder;

  void buildStationComboLists();
  void cleanupStationComboLists();

  UBahnIconStore _iconStore;

  void whatTimeIsNow();
  OperationTime::EDay _actOperationDay;
  unsigned int _actOperationHour;
  bool _useRealTime;
  
  void showStationInfo(Station *station);
  void showLineInfo(Line *line, RouteOfLine *routeOfLine);
  
  RouteItem *_selectedRouteItem;
  

private slots:
 void openMap();
 void closeMap(); 
 void exitApp();
 void mapInfo();
 void showLines();
 void showStations();
 void helpAbout();
 void findRoute();
 void switchStations();
 void showPassedStations();
 void setTime();
 void routeListDblClicked(QListViewItem *item);
 void routeListClicked(QListViewItem *item);
 void showConnections();
};


#endif
