/*
#########################################################################
# Subway Map
# ----------
# Copyright (c) 2004 by Dimitri Brukakis
# All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
#########################################################################
*/

#ifndef __XMLICONFILEPARSER_H__
#define __XMLICONFILEPARSER_H__

#include <subwaymap/baseclasses.h>
#include <subwaymap/XMLBase.h>
#include <subwaymap/Exceptions.h>


#include <qxml.h>


#include "UBahnIconStore.h"

// ---- XML Tag <icons>
const XMLString tag_icons      = "icons";

// XML Tag <info>
const XMLString tag_lineicons       = "lineicons";
const XMLString tag_icon			= "icon";
const XMLString attr_icon_line      = "line";
const XMLString attr_icon_file		= "file";


/*	============================================
	Base class for the IconFile parser.
	============================================ */

class XMLIconFileParser : public Object
{
 private:
  enum eParserState {
    none=0,
    start,         // got element <icons>
    in_lines,  	   // got element <lines>
	finished	   // got element </icons>
  };

 public:
  virtual ~XMLIconFileParser() {
  }
  virtual smerr_t parse(const char *filename, UBahnIconStore *iconStore);
  
 protected:
  XMLIconFileParser() : _exception(NULL) {}

  // -----------------------------------------------------------
  // abstract methods, that must be implemented by each subclass
  // -----------------------------------------------------------
  virtual smerr_t initParser()=0;
  virtual smerr_t parseDocument(const char *filename)=0;

  // -----------------------------------------------------------
  // interface for parsing an element
  // This method will be called by the subclass's element
  // parsing method.
  // -----------------------------------------------------------
  smerr_t parse_element(XMLElement& e);
  smerr_t finish_element(XMLElement& e);

  smerr_t parse_lineicon(XMLElement& e);		// parse one <icon> tag


 protected:
  UBahnIconStore *_iconStore;

  eParserState _state;

  void set_linecolumn(int line, int column) {
	_line=line;
	_column=column;
  }

  int _line;
  int _column;

 protected:
  // -----------------------------------
  // Error handling/Exception simulation
  // -----------------------------------
  smerr_t exception(const ExceptionBase *e) { 
	_exception=const_cast<ExceptionBase*>(e); 
	return _exception->get_extype();
  }

  ExceptionBase *_exception;

 public:
  ExceptionBase *get_exception() { return _exception; }
  void clean_exception() {
	if(_exception) delete _exception;
	_exception=NULL;
  }

};



/*	============================================
	Qt implementation of the IconFile parser.
	============================================ */

class XMLIconFileParser_Qt : public XMLIconFileParser, public QXmlContentHandler
{
 public:
  XMLIconFileParser_Qt() : XMLIconFileParser(), _locator(NULL) {}
  virtual ~XMLIconFileParser_Qt() {
	if(_locator) delete _locator;
  }

 protected:
  virtual smerr_t initParser();
  virtual smerr_t parseDocument(const char *filename);

 public:
  // ----------------------------------
  // Implementation of Qt SAX interface
  // ----------------------------------
  virtual bool startDocument();
  virtual bool endDocument();
  virtual bool startElement(const QString&, const QString&, const QString&,
							const QXmlAttributes&);
  virtual bool endElement(const QString&, const QString&, const QString&);
  virtual bool characters(const QString&);

  // pure virtual methods that must be defined
  void setDocumentLocator(QXmlLocator *) {}
  bool startPrefixMapping(const QString &, const QString &) { return TRUE; }
  bool endPrefixMapping(const QString &) { return TRUE; }
  bool ignorableWhitespace(const QString &) { return TRUE; }
  bool processingInstruction(const QString &, const QString &) { return TRUE; }
  bool skippedEntity(const QString &) { return TRUE; }
  QString errorString() { return "Hmm..."; }

 protected:
  QXmlLocator *_locator;
};





#endif
